@@ -199,7 +199,7 @@ static void sanitize_map(snd_pcm_chmap_t* map)
 	uint32_t i, j, pos;
 
 	for (i = 0; i < map->channels; i++) {
-		if (map->pos[i] < 0 || map->pos[i] > SND_CHMAP_LAST)
+		if (map->pos[i] > SND_CHMAP_LAST)
 			map->pos[i] = SND_CHMAP_UNKNOWN;
 
 		p = 1LL << map->pos[i];
@@ -314,7 +314,6 @@ spa_alsa_enum_format(struct state *state, int seq, uint32_t start, uint32_t num,
 		choice->body.type = SPA_CHOICE_Enum;
 	spa_pod_builder_pop(&b, &f[1]);
 
-
 	CHECK(snd_pcm_hw_params_get_rate_min(params, &min, &dir), "get_rate_min");
 	CHECK(snd_pcm_hw_params_get_rate_max(params, &max, &dir), "get_rate_max");
 
@@ -546,7 +545,7 @@ static int alsa_recover(struct state *state, int err)
 	snd_pcm_status_t *status;
 
 	snd_pcm_status_alloca(&status);
-	if ((res = snd_pcm_status(state->hndl, status)) < 0) {
+	if (SPA_UNLIKELY((res = snd_pcm_status(state->hndl, status)) < 0)) {
 		spa_log_error(state->log, NAME" %p: snd_pcm_status error: %s",
 				state, snd_strerror(res));
 		return res;
@@ -581,7 +580,7 @@ static int alsa_recover(struct state *state, int err)
 		break;
 	}
 
-	if ((res = snd_pcm_recover(state->hndl, err, true)) < 0) {
+	if (SPA_UNLIKELY((res = snd_pcm_recover(state->hndl, err, true)) < 0)) {
 		spa_log_error(state->log, NAME" %p: snd_pcm_recover error: %s",
 				state, snd_strerror(res));
 		return res;
@@ -609,11 +608,14 @@ static int get_status(struct state *state, snd_pcm_uframes_t *delay, snd_pcm_ufr
 	snd_pcm_sframes_t avail;
 	int res;
 
-	if ((avail = snd_pcm_avail(state->hndl)) < 0) {
+	if (SPA_UNLIKELY((avail = snd_pcm_avail(state->hndl)) < 0)) {
 		if ((res = alsa_recover(state, avail)) < 0)
 			return res;
-		if ((avail = snd_pcm_avail(state->hndl)) < 0)
-			return avail;
+		if ((avail = snd_pcm_avail(state->hndl)) < 0) {
+			spa_log_warn(state->log, NAME" %p: snd_pcm_avail after recover: %s",
+					state, snd_strerror(avail));
+			avail = state->threshold * 2;
+		}
 	} else {
 		state->alsa_recovering = false;
 	}
@@ -656,7 +658,7 @@ static int update_time(struct state *state, uint64_t nsec, snd_pcm_sframes_t del
 	else
 		err = (target + 128) - delay;
 
-	if (state->bw == 0.0) {
+	if (SPA_UNLIKELY(state->bw == 0.0)) {
 		set_loop(state, BW_MAX);
 		state->next_time = nsec;
 		state->base_time = nsec;
@@ -667,7 +669,7 @@ static int update_time(struct state *state, uint64_t nsec, snd_pcm_sframes_t del
 
 	corr = 1.0 - (state->z2 + state->z3);
 
-	if (state->last_threshold != state->threshold) {
+	if (SPA_UNLIKELY(state->last_threshold != state->threshold)) {
 		int32_t diff = (int32_t) (state->last_threshold - state->threshold);
 		spa_log_trace(state->log, NAME" %p: follower:%d quantum change %d",
 				state, follower, diff);
@@ -675,7 +677,7 @@ static int update_time(struct state *state, uint64_t nsec, snd_pcm_sframes_t del
 		state->last_threshold = state->threshold;
 	}
 
-	if ((state->next_time - state->base_time) > BW_PERIOD) {
+	if (SPA_UNLIKELY((state->next_time - state->base_time) > BW_PERIOD)) {
 		state->base_time = state->next_time;
 		if (state->bw == BW_MAX)
 			set_loop(state, BW_MED);
@@ -698,7 +700,7 @@ static int update_time(struct state *state, uint64_t nsec, snd_pcm_sframes_t del
 
 	state->next_time += state->threshold / corr * 1e9 / state->rate;
 
-	if (!follower && state->clock) {
+	if (SPA_LIKELY(!follower && state->clock)) {
 		state->clock->nsec = nsec;
 		state->clock->position += state->duration;
 		state->clock->duration = state->duration;
@@ -721,7 +723,7 @@ int spa_alsa_write(struct state *state, snd_pcm_uframes_t silence)
 	snd_pcm_uframes_t written, frames, offset, off, to_write, total_written;
 	int res;
 
-	if (state->position && state->duration != state->position->clock.duration) {
+	if (SPA_LIKELY(state->position && state->duration != state->position->clock.duration)) {
 		state->duration = state->position->clock.duration;
 		state->threshold = (state->duration * state->rate + state->rate_denom-1) / state->rate_denom;
 	}
@@ -730,16 +732,16 @@ int spa_alsa_write(struct state *state, snd_pcm_uframes_t silence)
 		uint64_t nsec;
 		snd_pcm_uframes_t delay, target;
 
-		if ((res = get_status(state, &delay, &target)) < 0)
+		if (SPA_UNLIKELY((res = get_status(state, &delay, &target)) < 0))
 			return res;
 
-		if (!state->alsa_recovering && delay > target + state->threshold) {
+		if (SPA_UNLIKELY(!state->alsa_recovering && delay > target + state->threshold)) {
 			spa_log_warn(state->log, NAME" %p: follower delay:%ld resync %f %f %f",
 					state, delay, state->z1, state->z2, state->z3);
 			init_loop(state);
 			state->alsa_sync = true;
 		}
-		if (state->alsa_sync) {
+		if (SPA_UNLIKELY(state->alsa_sync)) {
 			if (delay > target)
 				snd_pcm_rewind(state->hndl, delay - target);
 			else
@@ -750,14 +752,14 @@ int spa_alsa_write(struct state *state, snd_pcm_uframes_t silence)
 		}
 
 		nsec = state->position->clock.nsec;
-		if ((res = update_time(state, nsec, delay, target, true)) < 0)
+		if (SPA_UNLIKELY((res = update_time(state, nsec, delay, target, true)) < 0))
 			return res;
 	}
 
 	total_written = 0;
 again:
 	frames = state->buffer_frames;
-	if ((res = snd_pcm_mmap_begin(hndl, &my_areas, &offset, &frames)) < 0) {
+	if (SPA_UNLIKELY((res = snd_pcm_mmap_begin(hndl, &my_areas, &offset, &frames)) < 0)) {
 		spa_log_error(state->log, NAME" %p: snd_pcm_mmap_begin error: %s",
 				state, snd_strerror(res));
 		return res;
@@ -798,7 +800,7 @@ again:
 		l1 = n_bytes - l0;
 
 		spa_memcpy(dst, src + offs, l0);
-		if (l1 > 0)
+		if (SPA_UNLIKELY(l1 > 0))
 			spa_memcpy(dst + l0, src, l1);
 
 		state->ready_offset += n_bytes;
@@ -822,7 +824,7 @@ again:
 			silence = 0;
 	}
 
-	if (silence > 0) {
+	if (SPA_UNLIKELY(silence > 0)) {
 		spa_log_trace_fp(state->log, NAME" %p: silence %ld", state, silence);
 		snd_pcm_areas_silence(my_areas, off, state->channels, silence, state->format);
 		written += silence;
@@ -832,7 +834,7 @@ again:
 			state, offset, written, state->sample_count);
 	total_written += written;
 
-	if ((res = snd_pcm_mmap_commit(hndl, offset, written)) < 0) {
+	if (SPA_UNLIKELY((res = snd_pcm_mmap_commit(hndl, offset, written)) < 0)) {
 		spa_log_error(state->log, NAME" %p: snd_pcm_mmap_commit error: %s",
 				state, snd_strerror(res));
 		if (res != -EPIPE && res != -ESTRPIPE)
@@ -844,7 +846,7 @@ again:
 
 	state->sample_count += total_written;
 
-	if (!state->alsa_started && total_written > 0) {
+	if (SPA_UNLIKELY(!state->alsa_started && total_written > 0)) {
 		spa_log_trace(state->log, NAME" %p: snd_pcm_start %lu", state, written);
 		if ((res = snd_pcm_start(hndl)) < 0) {
 			spa_log_error(state->log, NAME" %p: snd_pcm_start: %s",
@@ -856,6 +858,17 @@ again:
 	return 0;
 }
 
+void spa_alsa_recycle_buffer(struct state *this, uint32_t buffer_id)
+{
+	struct buffer *b = &this->buffers[buffer_id];
+
+	if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_OUT)) {
+		spa_log_trace_fp(this->log, NAME " %p: recycle buffer %u", this, buffer_id);
+		spa_list_append(&this->free, &b->link);
+		SPA_FLAG_CLEAR(b->flags, BUFFER_FLAG_OUT);
+	}
+}
+
 static snd_pcm_uframes_t
 push_frames(struct state *state,
 	    const snd_pcm_channel_area_t *my_areas,
@@ -907,7 +920,6 @@ push_frames(struct state *state,
 		d[0].chunk->size = n_bytes;
 		d[0].chunk->stride = state->frame_size;
 
-		SPA_FLAG_SET(b->flags, BUFFER_FLAG_OUT);
 		spa_list_append(&state->ready, &b->link);
 	}
 	return total_frames - keep;
@@ -1009,13 +1021,13 @@ static int handle_play(struct state *state, uint64_t nsec,
 {
 	int res;
 
-	if (delay > target + state->last_threshold) {
+	if (SPA_UNLIKELY(delay > target + state->last_threshold)) {
 		spa_log_trace(state->log, NAME" %p: early wakeup %ld %ld", state, delay, target);
 		state->next_time = nsec + (delay - target) * SPA_NSEC_PER_SEC / state->rate;
 		return -EAGAIN;
 	}
 
-	if ((res = update_time(state, nsec, delay, target, false)) < 0)
+	if (SPA_UNLIKELY((res = update_time(state, nsec, delay, target, false)) < 0))
 		return res;
 
 	if (spa_list_is_empty(&state->ready)) {
@@ -1052,15 +1064,22 @@ static int handle_capture(struct state *state, uint64_t nsec,
 	if ((res = spa_alsa_read(state, target)) < 0)
 		return res;
 
-	if (!spa_list_is_empty(&state->ready)) {
-		io = state->io;
-		if (io != NULL && io->status != SPA_STATUS_HAVE_DATA) {
-			struct buffer *b = spa_list_first(&state->ready, struct buffer, link);
-			spa_list_remove(&b->link);
+	if (spa_list_is_empty(&state->ready))
+		return 0;
 
-			io->buffer_id = b->id;
-			io->status = SPA_STATUS_HAVE_DATA;
-		}
+	io = state->io;
+	if (io != NULL && io->status != SPA_STATUS_HAVE_DATA) {
+		struct buffer *b;
+
+		if (io->buffer_id < state->n_buffers)
+			spa_alsa_recycle_buffer(state, io->buffer_id);
+
+		b = spa_list_first(&state->ready, struct buffer, link);
+		spa_list_remove(&b->link);
+		SPA_FLAG_SET(b->flags, BUFFER_FLAG_OUT);
+
+		io->buffer_id = b->id;
+		io->status = SPA_STATUS_HAVE_DATA;
 	}
 	spa_node_call_ready(&state->callbacks, SPA_STATUS_HAVE_DATA);
 	return 0;
@@ -1073,15 +1092,15 @@ static void alsa_on_timeout_event(struct spa_source *source)
 	uint64_t expire;
 	int res;
 
-	if (state->started && spa_system_timerfd_read(state->data_system, state->timerfd, &expire) < 0)
+	if (SPA_UNLIKELY(state->started && spa_system_timerfd_read(state->data_system, state->timerfd, &expire) < 0))
 		spa_log_warn(state->log, NAME" %p: error reading timerfd: %m", state);
 
-	if (state->position) {
+	if (SPA_LIKELY(state->position)) {
 		state->duration = state->position->clock.duration;
 		state->threshold = (state->duration * state->rate + state->rate_denom-1) / state->rate_denom;
 	}
 
-	if ((res = get_status(state, &delay, &target)) < 0)
+	if (SPA_UNLIKELY((res = get_status(state, &delay, &target)) < 0))
 		return;
 
 	state->current_time = state->next_time;
