@@ -1,20 +1,25 @@
 /* PipeWire
- * Copyright (C) 2017 Wim Taymans <wim.taymans@gmail.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * Copyright © 2018 Wim Taymans
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include <string.h>
@@ -27,29 +32,18 @@
 
 #include <dbus/dbus.h>
 
-#include <spa/support/type-map.h>
+#include <spa/utils/type.h>
+#include <spa/utils/names.h>
 #include <spa/support/log.h>
 #include <spa/support/plugin.h>
 #include <spa/support/dbus.h>
 
 #define NAME "dbus"
 
-struct type {
-	uint32_t dbus;
-};
-
-static inline void init_type(struct type *type, struct spa_type_map *map)
-{
-	type->dbus = spa_type_map_get_id(map, SPA_TYPE__DBus);
-}
-
 struct impl {
 	struct spa_handle handle;
 	struct spa_dbus dbus;
 
-	struct type type;
-
-	struct spa_type_map *map;
 	struct spa_log *log;
 	struct spa_loop_utils *utils;
 
@@ -83,9 +77,9 @@ static void dispatch_status(DBusConnection *conn, DBusDispatchStatus status, voi
 			status == DBUS_DISPATCH_COMPLETE ? false : true);
 }
 
-static inline enum spa_io dbus_to_io(DBusWatch *watch)
+static inline uint32_t dbus_to_io(DBusWatch *watch)
 {
-	enum spa_io mask;
+	uint32_t mask;
 	unsigned int flags;
 
 	/* no watch flags for disabled watches */
@@ -103,7 +97,7 @@ static inline enum spa_io dbus_to_io(DBusWatch *watch)
 	return mask;
 }
 
-static inline unsigned int io_to_dbus(enum spa_io mask)
+static inline unsigned int io_to_dbus(uint32_t mask)
 {
 	unsigned int flags = 0;
 
@@ -119,7 +113,7 @@ static inline unsigned int io_to_dbus(enum spa_io mask)
 }
 
 static void
-handle_io_event(void *userdata, int fd, enum spa_io mask)
+handle_io_event(void *userdata, int fd, uint32_t mask)
 {
 	DBusWatch *watch = userdata;
 
@@ -287,24 +281,27 @@ static const struct spa_dbus_connection impl_connection = {
 };
 
 static struct spa_dbus_connection *
-impl_get_connection(struct spa_dbus *dbus,
+impl_get_connection(void *object,
 		    enum spa_dbus_type type)
 {
-        struct impl *impl = SPA_CONTAINER_OF(dbus, struct impl, dbus);
+        struct impl *impl = object;
 	struct connection *conn;
         DBusError error;
+	int res;
 
 	dbus_error_init(&error);
 
 	conn = calloc(1, sizeof(struct connection));
 	conn->this = impl_connection;
 	conn->impl = impl;
-	conn->conn = dbus_bus_get_private((DBusBusType)type, &error);
+	conn->conn = dbus_bus_get_private(type, &error);
 	if (conn->conn == NULL)
 		goto error;
 
 	conn->dispatch_event = spa_loop_utils_add_idle(impl->utils,
 						false, dispatch_cb, conn);
+	if (conn->dispatch_event == NULL)
+		goto no_event;
 
 	dbus_connection_set_exit_on_disconnect(conn->conn, false);
 	dbus_connection_set_dispatch_status_function(conn->conn, dispatch_status, conn, NULL);
@@ -321,16 +318,28 @@ impl_get_connection(struct spa_dbus *dbus,
       error:
 	spa_log_error(impl->log, "Failed to connect to system bus: %s", error.message);
 	dbus_error_free(&error);
+	res = -ECONNREFUSED;
+	goto out_free;
+      no_event:
+	res = -errno;
+	spa_log_error(impl->log, "Failed to create idle event: %m");
+	goto out_unref_dbus;
+
+      out_unref_dbus:
+	dbus_connection_close(conn->conn);
+	dbus_connection_unref(conn->conn);
+      out_free:
 	free(conn);
+	errno = -res;
 	return NULL;
 }
 
-static const struct spa_dbus impl_dbus = {
-	SPA_VERSION_DBUS,
-	impl_get_connection,
+static const struct spa_dbus_methods impl_dbus = {
+	SPA_VERSION_DBUS_METHODS,
+	.get_connection = impl_get_connection,
 };
 
-static int impl_get_interface(struct spa_handle *handle, uint32_t interface_id, void **interface)
+static int impl_get_interface(struct spa_handle *handle, const char *type, void **interface)
 {
 	struct impl *this;
 
@@ -339,7 +348,7 @@ static int impl_get_interface(struct spa_handle *handle, uint32_t interface_id,
 
 	this = (struct impl *) handle;
 
-	if (interface_id == this->type.dbus)
+	if (strcmp(type, SPA_TYPE_INTERFACE_DBus) == 0)
 		*interface = &this->dbus;
 	else
 		return -ENOENT;
@@ -353,6 +362,13 @@ static int impl_clear(struct spa_handle *handle)
 	return 0;
 }
 
+static size_t
+impl_get_size(const struct spa_handle_factory *factory,
+	      const struct spa_dict *params)
+{
+	return sizeof(struct impl);
+}
+
 static int
 impl_init(const struct spa_handle_factory *factory,
 	  struct spa_handle *handle,
@@ -361,7 +377,6 @@ impl_init(const struct spa_handle_factory *factory,
 	  uint32_t n_support)
 {
 	struct impl *this;
-	uint32_t i;
 
 	spa_return_val_if_fail(factory != NULL, -EINVAL);
 	spa_return_val_if_fail(handle != NULL, -EINVAL);
@@ -372,25 +387,18 @@ impl_init(const struct spa_handle_factory *factory,
 	this = (struct impl *) handle;
 	spa_list_init(&this->connection_list);
 
-	this->dbus = impl_dbus;
+	this->dbus.iface = SPA_INTERFACE_INIT(
+			SPA_TYPE_INTERFACE_DBus,
+			SPA_VERSION_DBUS,
+			&impl_dbus, this);
+
+	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
+	this->utils = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_LoopUtils);
 
-	for (i = 0; i < n_support; i++) {
-		if (strcmp(support[i].type, SPA_TYPE__TypeMap) == 0)
-			this->map = support[i].data;
-		else if (strcmp(support[i].type, SPA_TYPE__Log) == 0)
-			this->log = support[i].data;
-		else if (strcmp(support[i].type, SPA_TYPE__LoopUtils) == 0)
-			this->utils = support[i].data;
-	}
-	if (this->map == NULL) {
-		spa_log_error(this->log, "a type-map is needed");
-		return -EINVAL;
-	}
 	if (this->utils == NULL) {
 		spa_log_error(this->log, "a LoopUtils is needed");
 		return -EINVAL;
 	}
-	init_type(&this->type, this->map);
 
 	spa_log_debug(this->log, NAME " %p: initialized", this);
 
@@ -398,7 +406,7 @@ impl_init(const struct spa_handle_factory *factory,
 }
 
 static const struct spa_interface_info impl_interfaces[] = {
-	{SPA_TYPE__DBus,},
+	{SPA_TYPE_INTERFACE_DBus,},
 };
 
 static int
@@ -424,16 +432,15 @@ impl_enum_interface_info(const struct spa_handle_factory *factory,
 
 static const struct spa_handle_factory dbus_factory = {
 	SPA_VERSION_HANDLE_FACTORY,
-	NAME,
+	SPA_NAME_SUPPORT_DBUS,
 	NULL,
-	sizeof(struct impl),
+	impl_get_size,
 	impl_init,
 	impl_enum_interface_info,
 };
 
 SPA_EXPORT
-int
-spa_handle_factory_enum(const struct spa_handle_factory **factory, uint32_t *index)
+int spa_handle_factory_enum(const struct spa_handle_factory **factory, uint32_t *index)
 {
 	spa_return_val_if_fail(factory != NULL, -EINVAL);
 	spa_return_val_if_fail(index != NULL, -EINVAL);
