@@ -51,8 +51,6 @@
 
 #define DEFAULT_LIVE true
 
-#define MAX_HEIGHT 1024
-
 struct props {
 	bool live;
 };
@@ -327,7 +325,6 @@ static int make_buffer(struct impl *this)
 			b->h->dts_offset = 0;
 		}
 
-		SPA_FLAG_SET(b->flags, BUFFER_FLAG_OUT);
 		spa_list_append(&port->ready, &b->link);
 
 		res = SPA_STATUS_HAVE_DATA;
@@ -378,6 +375,7 @@ static void on_output(struct spa_source *source)
 	if (!spa_list_is_empty(&port->ready)) {
 		struct buffer *b = spa_list_first(&port->ready, struct buffer, link);
 		spa_list_remove(&b->link);
+		SPA_FLAG_SET(b->flags, BUFFER_FLAG_OUT);
 
 		io->buffer_id = b->id;
 		io->status = SPA_STATUS_HAVE_DATA;
@@ -588,11 +586,17 @@ impl_node_port_enum_params(void *object, int seq,
 		if (result.index > 0)
 			return 0;
 
+		spa_log_debug(this->log, NAME" %p: %dx%d stride %d", this,
+				this->position->video.size.width,
+				this->position->video.size.height,
+				this->position->video.stride);
+
 		param = spa_pod_builder_add_object(&b,
 			SPA_TYPE_OBJECT_ParamBuffers, id,
 			SPA_PARAM_BUFFERS_buffers, SPA_POD_CHOICE_RANGE_Int(2, 1, MAX_BUFFERS),
 			SPA_PARAM_BUFFERS_blocks,  SPA_POD_Int(1),
-			SPA_PARAM_BUFFERS_size,    SPA_POD_Int(this->position->video.stride * MAX_HEIGHT),
+			SPA_PARAM_BUFFERS_size,    SPA_POD_Int(this->position->video.stride *
+								this->position->video.size.height),
 			SPA_PARAM_BUFFERS_stride,  SPA_POD_Int(this->position->video.stride),
 			SPA_PARAM_BUFFERS_align,   SPA_POD_Int(16));
 		break;
