@@ -1,20 +1,25 @@
 /* GStreamer
- * Copyright (C) <2015> Wim Taymans <wim.taymans@gmail.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * Copyright © 2018 Wim Taymans
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 /**
@@ -40,6 +45,8 @@
 #include <sys/socket.h>
 #include <unistd.h>
 
+#include <spa/pod/builder.h>
+
 #include <gst/net/gstnetclientclock.h>
 #include <gst/allocators/gstfdmemory.h>
 #include <gst/allocators/gstdmabuf.h>
@@ -201,12 +208,9 @@ gst_pipewire_src_finalize (GObject * object)
 
   clear_queue (pwsrc);
 
-  pw_core_destroy (pwsrc->core);
-  pwsrc->core = NULL;
-  pwsrc->type = NULL;
-  pw_thread_loop_destroy (pwsrc->main_loop);
-  pwsrc->main_loop = NULL;
-  pw_loop_destroy (pwsrc->loop);
+  pw_context_destroy (pwsrc->context);
+  pwsrc->context = NULL;
+  pw_thread_loop_destroy (pwsrc->loop);
   pwsrc->loop = NULL;
 
   if (pwsrc->properties)
@@ -320,15 +324,12 @@ gst_pipewire_src_init (GstPipeWireSrc * src)
 
   g_queue_init (&src->queue);
 
-  src->client_name = pw_get_client_name ();
+  src->client_name = g_strdup(pw_get_client_name ());
 
   src->pool =  gst_pipewire_pool_new ();
-  src->loop = pw_loop_new (NULL);
-  src->main_loop = pw_thread_loop_new (src->loop, "pipewire-main-loop");
-  src->core = pw_core_new (src->loop, NULL);
-  src->type = pw_core_get_type (src->core);
-  src->pool->t = src->type;
-  GST_DEBUG ("loop %p, mainloop %p", src->loop, src->main_loop);
+  src->loop = pw_thread_loop_new ("pipewire-main-loop", NULL);
+  src->context = pw_context_new (pw_thread_loop_get_loop(src->loop), NULL, 0);
+  GST_DEBUG ("loop %p context %p", src->loop, src->context);
 
 }
 
@@ -345,9 +346,9 @@ buffer_recycle (GstMiniObject *obj)
   src = data->owner;
 
   GST_LOG_OBJECT (obj, "recycle buffer");
-  pw_thread_loop_lock (src->main_loop);
+  pw_thread_loop_lock (src->loop);
   pw_stream_queue_buffer (src->stream, data->b);
-  pw_thread_loop_unlock (src->main_loop);
+  pw_thread_loop_unlock (src->loop);
 
   return FALSE;
 }
@@ -358,7 +359,7 @@ on_add_buffer (void *_data, struct pw_buffer *b)
   GstPipeWireSrc *pwsrc = _data;
   GstPipeWirePoolData *data;
 
-  GST_LOG_OBJECT (pwsrc, "add buffer");
+  GST_DEBUG_OBJECT (pwsrc, "add buffer");
   gst_pipewire_pool_wrap_buffer (pwsrc->pool, b);
   data = b->user_data;
   data->owner = pwsrc;
@@ -373,7 +374,7 @@ on_remove_buffer (void *_data, struct pw_buffer *b)
   GstBuffer *buf = data->buf;
   GList *walk;
 
-  GST_LOG_OBJECT (pwsrc, "remove buffer %p", buf);
+  GST_DEBUG_OBJECT (pwsrc, "remove buffer %p", buf);
 
   GST_MINI_OBJECT_CAST (buf)->dispose = NULL;
 
@@ -402,7 +403,7 @@ on_process (void *_data)
 
   b = pw_stream_dequeue_buffer (pwsrc->stream);
   if (b == NULL)
-	  return;
+          return;
 
   data = b->user_data;
   buf = data->buf;
@@ -414,10 +415,10 @@ on_process (void *_data)
 
   h = data->header;
   if (h) {
-    GST_INFO ("pts %" G_GUINT64_FORMAT ", dts_offset %"G_GUINT64_FORMAT, h->pts, h->dts_offset);
+    GST_LOG_OBJECT (pwsrc, "pts %" G_GUINT64_FORMAT ", dts_offset %" G_GUINT64_FORMAT, h->pts, h->dts_offset);
 
     if (GST_CLOCK_TIME_IS_VALID (h->pts)) {
-      GST_BUFFER_PTS (buf) = h->pts;
+      GST_BUFFER_PTS (buf) = h->pts + GST_PIPEWIRE_CLOCK (pwsrc->clock)->time_offset;
       if (GST_BUFFER_PTS (buf) + h->dts_offset > 0)
         GST_BUFFER_DTS (buf) = GST_BUFFER_PTS (buf) + h->dts_offset;
     }
@@ -431,18 +432,17 @@ on_process (void *_data)
     mem->offset += data->offset;
   }
 
-
   gst_buffer_ref (buf);
   g_queue_push_tail (&pwsrc->queue, buf);
 
-  pw_thread_loop_signal (pwsrc->main_loop, FALSE);
+  pw_thread_loop_signal (pwsrc->loop, FALSE);
   return;
 }
 
 static void
 on_state_changed (void *data,
-		  enum pw_stream_state old,
-		  enum pw_stream_state state, const char *error)
+                  enum pw_stream_state old,
+                  enum pw_stream_state state, const char *error)
 {
   GstPipeWireSrc *pwsrc = data;
 
@@ -451,8 +451,6 @@ on_state_changed (void *data,
   switch (state) {
     case PW_STREAM_STATE_UNCONNECTED:
     case PW_STREAM_STATE_CONNECTING:
-    case PW_STREAM_STATE_CONFIGURE:
-    case PW_STREAM_STATE_READY:
     case PW_STREAM_STATE_PAUSED:
     case PW_STREAM_STATE_STREAMING:
       break;
@@ -461,7 +459,7 @@ on_state_changed (void *data,
           ("stream error: %s", error), (NULL));
       break;
   }
-  pw_thread_loop_signal (pwsrc->main_loop, FALSE);
+  pw_thread_loop_signal (pwsrc->loop, FALSE);
 }
 
 static void
@@ -471,13 +469,13 @@ parse_stream_properties (GstPipeWireSrc *pwsrc, const struct pw_properties *prop
   gboolean is_live;
 
   GST_OBJECT_LOCK (pwsrc);
-  var = pw_properties_get (props, PW_STREAM_PROP_IS_LIVE);
+  var = pw_properties_get (props, PW_KEY_STREAM_IS_LIVE);
   is_live = pwsrc->is_live = var ? pw_properties_parse_bool(var) : FALSE;
 
-  var = pw_properties_get (props, PW_STREAM_PROP_LATENCY_MIN);
+  var = pw_properties_get (props, PW_KEY_STREAM_LATENCY_MIN);
   pwsrc->min_latency = var ? (GstClockTime) atoi (var) : 0;
 
-  var = pw_properties_get (props, PW_STREAM_PROP_LATENCY_MAX);
+  var = pw_properties_get (props, PW_KEY_STREAM_LATENCY_MAX);
   pwsrc->max_latency = var ? (GstClockTime) atoi (var) : GST_CLOCK_TIME_NONE;
   GST_OBJECT_UNLOCK (pwsrc);
 
@@ -490,7 +488,7 @@ static gboolean
 gst_pipewire_src_stream_start (GstPipeWireSrc *pwsrc)
 {
   const char *error = NULL;
-  pw_thread_loop_lock (pwsrc->main_loop);
+  pw_thread_loop_lock (pwsrc->loop);
   GST_DEBUG_OBJECT (pwsrc, "doing stream start");
   while (TRUE) {
     enum pw_stream_state state = pw_stream_get_state (pwsrc->stream, &error);
@@ -502,24 +500,21 @@ gst_pipewire_src_stream_start (GstPipeWireSrc *pwsrc)
     if (state == PW_STREAM_STATE_ERROR)
       goto start_error;
 
-    if (pw_remote_get_state(pwsrc->remote, &error) == PW_REMOTE_STATE_ERROR)
-      goto start_error;
-
-    pw_thread_loop_wait (pwsrc->main_loop);
+    pw_thread_loop_wait (pwsrc->loop);
   }
 
   parse_stream_properties (pwsrc, pw_stream_get_properties (pwsrc->stream));
   GST_DEBUG_OBJECT (pwsrc, "signal started");
   pwsrc->started = TRUE;
-  pw_thread_loop_signal (pwsrc->main_loop, FALSE);
-  pw_thread_loop_unlock (pwsrc->main_loop);
+  pw_thread_loop_signal (pwsrc->loop, FALSE);
+  pw_thread_loop_unlock (pwsrc->loop);
 
   return TRUE;
 
 start_error:
   {
     GST_DEBUG_OBJECT (pwsrc, "error starting stream: %s", error);
-    pw_thread_loop_unlock (pwsrc->main_loop);
+    pw_thread_loop_unlock (pwsrc->loop);
     return FALSE;
   }
 }
@@ -530,7 +525,7 @@ wait_negotiated (GstPipeWireSrc *this)
   enum pw_stream_state state;
   const char *error = NULL;
 
-  pw_thread_loop_lock (this->main_loop);
+  pw_thread_loop_lock (this->loop);
   while (TRUE) {
     state = pw_stream_get_state (this->stream, &error);
 
@@ -540,16 +535,13 @@ wait_negotiated (GstPipeWireSrc *this)
     if (state == PW_STREAM_STATE_ERROR)
       break;
 
-    if (pw_remote_get_state(this->remote, &error) == PW_REMOTE_STATE_ERROR)
-      break;
-
     if (this->started)
       break;
 
-    pw_thread_loop_wait (this->main_loop);
+    pw_thread_loop_wait (this->loop);
   }
   GST_DEBUG_OBJECT (this, "got started signal");
-  pw_thread_loop_unlock (this->main_loop);
+  pw_thread_loop_unlock (this->loop);
 
   return state;
 }
@@ -592,11 +584,11 @@ gst_pipewire_src_negotiate (GstBaseSrc * basesrc)
   GST_DEBUG_OBJECT (basesrc, "have common caps: %" GST_PTR_FORMAT, caps);
 
   /* open a connection with these caps */
-  possible = gst_caps_to_format_all (caps, pwsrc->type->param.idEnumFormat, pwsrc->type->map);
+  possible = gst_caps_to_format_all (caps, SPA_PARAM_EnumFormat);
   gst_caps_unref (caps);
 
   /* first disconnect */
-  pw_thread_loop_lock (pwsrc->main_loop);
+  pw_thread_loop_lock (pwsrc->loop);
   if (pw_stream_get_state(pwsrc->stream, &error) != PW_STREAM_STATE_UNCONNECTED) {
     GST_DEBUG_OBJECT (basesrc, "disconnect capture");
     pw_stream_disconnect (pwsrc->stream);
@@ -612,14 +604,14 @@ gst_pipewire_src_negotiate (GstBaseSrc * basesrc)
         goto connect_error;
       }
 
-      pw_thread_loop_wait (pwsrc->main_loop);
+      pw_thread_loop_wait (pwsrc->loop);
     }
   }
 
   GST_DEBUG_OBJECT (basesrc, "connect capture with path %s", pwsrc->path);
   pw_stream_connect (pwsrc->stream,
                      PW_DIRECTION_INPUT,
-                     pwsrc->path,
+                     pwsrc->path ? (uint32_t)atoi(pwsrc->path) : PW_ID_ANY,
                      PW_STREAM_FLAG_AUTOCONNECT,
                      (const struct spa_pod **)possible->pdata,
                      possible->len);
@@ -636,12 +628,9 @@ gst_pipewire_src_negotiate (GstBaseSrc * basesrc)
     if (state == PW_STREAM_STATE_ERROR)
       goto connect_error;
 
-    if (pw_remote_get_state(pwsrc->remote, &error) == PW_REMOTE_STATE_ERROR)
-      goto connect_error;
-
-    pw_thread_loop_wait (pwsrc->main_loop);
+    pw_thread_loop_wait (pwsrc->loop);
   }
-  pw_thread_loop_unlock (pwsrc->main_loop);
+  pw_thread_loop_unlock (pwsrc->loop);
 
   result = gst_pipewire_src_stream_start (pwsrc);
 
@@ -676,30 +665,27 @@ no_common_caps:
   }
 connect_error:
   {
-    pw_thread_loop_unlock (pwsrc->main_loop);
+    pw_thread_loop_unlock (pwsrc->loop);
     return FALSE;
   }
 }
 
-#define SPA_PROP_RANGE(min,max)	2,min,max
-
 static void
-on_format_changed (void *data,
-                   const struct spa_pod *format)
+on_param_changed (void *data, uint32_t id,
+                   const struct spa_pod *param)
 {
   GstPipeWireSrc *pwsrc = data;
   GstCaps *caps;
   gboolean res;
-  struct pw_core *core = pwsrc->core;
-  struct pw_type *t = pw_core_get_type(core);
 
-  if (format == NULL) {
+  if (param == NULL || id != SPA_PARAM_Format) {
     GST_DEBUG_OBJECT (pwsrc, "clear format");
-    pw_stream_finish_format (pwsrc->stream, 0, NULL, 0);
     return;
   }
 
-  caps = gst_caps_from_format (format, t->map);
+  gst_pipewire_clock_reset (GST_PIPEWIRE_CLOCK (pwsrc->clock), 0);
+
+  caps = gst_caps_from_format (param);
   GST_DEBUG_OBJECT (pwsrc, "we got format %" GST_PTR_FORMAT, caps);
   res = gst_base_src_set_caps (GST_BASE_SRC (pwsrc), caps);
   gst_caps_unref (caps);
@@ -710,23 +696,24 @@ on_format_changed (void *data,
     uint8_t buffer[512];
 
     spa_pod_builder_init (&b, buffer, sizeof (buffer));
-    params[0] = spa_pod_builder_object (&b,
-	t->param.idBuffers, t->param_buffers.Buffers,
-	":", t->param_buffers.size,    "ir", 0,  SPA_PROP_RANGE(0, INT32_MAX),
-	":", t->param_buffers.stride,  "ir", 0,  SPA_PROP_RANGE(0, INT32_MAX),
-	":", t->param_buffers.buffers, "ir", 16, SPA_PROP_RANGE(1, INT32_MAX),
-	":", t->param_buffers.align,   "i", 16);
+    params[0] = spa_pod_builder_add_object (&b,
+        SPA_TYPE_OBJECT_ParamBuffers, SPA_PARAM_Buffers,
+        SPA_PARAM_BUFFERS_buffers, SPA_POD_CHOICE_RANGE_Int(16, 1, INT32_MAX),
+        SPA_PARAM_BUFFERS_blocks,  SPA_POD_CHOICE_RANGE_Int(0, 1, INT32_MAX),
+        SPA_PARAM_BUFFERS_size,    SPA_POD_CHOICE_RANGE_Int(0, 0, INT32_MAX),
+        SPA_PARAM_BUFFERS_stride,  SPA_POD_CHOICE_RANGE_Int(0, 0, INT32_MAX),
+        SPA_PARAM_BUFFERS_align,   SPA_POD_Int(16));
 
-    params[1] = spa_pod_builder_object (&b,
-	t->param.idMeta, t->param_meta.Meta,
-        ":", t->param_meta.type, "I", t->meta.Header,
-        ":", t->param_meta.size, "i", sizeof (struct spa_meta_header));
+    params[1] = spa_pod_builder_add_object (&b,
+        SPA_TYPE_OBJECT_ParamMeta, SPA_PARAM_Meta,
+        SPA_PARAM_META_type, SPA_POD_Id(SPA_META_Header),
+        SPA_PARAM_META_size, SPA_POD_Int(sizeof (struct spa_meta_header)));
 
     GST_DEBUG_OBJECT (pwsrc, "doing finish format");
-    pw_stream_finish_format (pwsrc->stream, 0, params, 2);
+    pw_stream_update_params (pwsrc->stream, params, 2);
   } else {
     GST_WARNING_OBJECT (pwsrc, "finish format with error");
-    pw_stream_finish_format (pwsrc->stream, -EINVAL, NULL, 0);
+    pw_stream_set_error (pwsrc->stream, -EINVAL, "unhandled format");
   }
 }
 
@@ -735,11 +722,11 @@ gst_pipewire_src_unlock (GstBaseSrc * basesrc)
 {
   GstPipeWireSrc *pwsrc = GST_PIPEWIRE_SRC (basesrc);
 
-  pw_thread_loop_lock (pwsrc->main_loop);
+  pw_thread_loop_lock (pwsrc->loop);
   GST_DEBUG_OBJECT (pwsrc, "setting flushing");
   pwsrc->flushing = TRUE;
-  pw_thread_loop_signal (pwsrc->main_loop, FALSE);
-  pw_thread_loop_unlock (pwsrc->main_loop);
+  pw_thread_loop_signal (pwsrc->loop, FALSE);
+  pw_thread_loop_unlock (pwsrc->loop);
 
   return TRUE;
 }
@@ -749,10 +736,10 @@ gst_pipewire_src_unlock_stop (GstBaseSrc * basesrc)
 {
   GstPipeWireSrc *pwsrc = GST_PIPEWIRE_SRC (basesrc);
 
-  pw_thread_loop_lock (pwsrc->main_loop);
+  pw_thread_loop_lock (pwsrc->loop);
   GST_DEBUG_OBJECT (pwsrc, "unsetting flushing");
   pwsrc->flushing = FALSE;
-  pw_thread_loop_unlock (pwsrc->main_loop);
+  pw_thread_loop_unlock (pwsrc->loop);
 
   return TRUE;
 }
@@ -772,25 +759,6 @@ gst_pipewire_src_event (GstBaseSrc * src, GstEvent * event)
         gst_video_event_parse_upstream_force_key_unit (event,
                 &running_time, &all_headers, &count);
 
-#if 0
-        pw_buffer_builder_init (&b);
-
-        refresh.last_id = 0;
-        refresh.request_type = all_headers ? 1 : 0;
-        refresh.pts = running_time;
-        pw_buffer_builder_add_refresh_request (&b, &refresh);
-
-        pw_buffer_builder_end (&b, &pbuf);
-
-        GST_OBJECT_LOCK (pwsrc);
-        if (pwsrc->stream_state == PW_STREAM_STATE_STREAMING) {
-          GST_DEBUG_OBJECT (pwsrc, "send refresh request");
-          pw_stream_send_buffer (pwsrc->stream, &pbuf);
-        }
-        GST_OBJECT_UNLOCK (pwsrc);
-
-        pw_buffer_unref (&pbuf);
-#endif
         res = TRUE;
       } else {
         res = GST_BASE_SRC_CLASS (parent_class)->event (src, event);
@@ -840,7 +808,7 @@ gst_pipewire_src_create (GstPushSrc * psrc, GstBuffer ** buffer)
   if (!pwsrc->negotiated)
     goto not_negotiated;
 
-  pw_thread_loop_lock (pwsrc->main_loop);
+  pw_thread_loop_lock (pwsrc->loop);
   while (TRUE) {
     enum pw_stream_state state;
 
@@ -858,13 +826,13 @@ gst_pipewire_src_create (GstPushSrc * psrc, GstBuffer ** buffer)
       goto streaming_stopped;
 
     buf = g_queue_pop_head (&pwsrc->queue);
-    GST_DEBUG ("popped buffer %p", buf);
+    GST_LOG_OBJECT (pwsrc, "popped buffer %p", buf);
     if (buf != NULL)
       break;
 
-    pw_thread_loop_wait (pwsrc->main_loop);
+    pw_thread_loop_wait (pwsrc->loop);
   }
-  pw_thread_loop_unlock (pwsrc->main_loop);
+  pw_thread_loop_unlock (pwsrc->loop);
 
   gst_buffer_unref (buf);
 
@@ -888,8 +856,8 @@ gst_pipewire_src_create (GstPushSrc * psrc, GstBuffer ** buffer)
   if (GST_CLOCK_TIME_IS_VALID (dts))
     dts = (dts >= base_time ? dts - base_time : 0);
 
-  GST_INFO ("pts %" G_GUINT64_FORMAT ", dts %"G_GUINT64_FORMAT
-      ", base-time %"GST_TIME_FORMAT" -> %"GST_TIME_FORMAT", %"GST_TIME_FORMAT,
+  GST_LOG_OBJECT (pwsrc,
+      "pts %" G_GUINT64_FORMAT ", dts %" G_GUINT64_FORMAT ", base-time %" GST_TIME_FORMAT " -> %" GST_TIME_FORMAT ", %" GST_TIME_FORMAT,
       GST_BUFFER_PTS (*buffer), GST_BUFFER_DTS (*buffer), GST_TIME_ARGS (base_time),
       GST_TIME_ARGS (pts), GST_TIME_ARGS (dts));
 
@@ -904,12 +872,12 @@ not_negotiated:
   }
 streaming_error:
   {
-    pw_thread_loop_unlock (pwsrc->main_loop);
+    pw_thread_loop_unlock (pwsrc->loop);
     return GST_FLOW_ERROR;
   }
 streaming_stopped:
   {
-    pw_thread_loop_unlock (pwsrc->main_loop);
+    pw_thread_loop_unlock (pwsrc->loop);
     return GST_FLOW_FLUSHING;
   }
 }
@@ -927,33 +895,13 @@ gst_pipewire_src_stop (GstBaseSrc * basesrc)
 
   pwsrc = GST_PIPEWIRE_SRC (basesrc);
 
-  pw_thread_loop_lock (pwsrc->main_loop);
+  pw_thread_loop_lock (pwsrc->loop);
   clear_queue (pwsrc);
-  pw_thread_loop_unlock (pwsrc->main_loop);
+  pw_thread_loop_unlock (pwsrc->loop);
 
   return TRUE;
 }
 
-static void
-on_remote_state_changed (void *data, enum pw_remote_state old, enum pw_remote_state state, const char *error)
-{
-  GstPipeWireSrc *pwsrc = data;
-
-  GST_DEBUG ("got remote state %s", pw_remote_state_as_string (state));
-
-  switch (state) {
-    case PW_REMOTE_STATE_UNCONNECTED:
-    case PW_REMOTE_STATE_CONNECTING:
-    case PW_REMOTE_STATE_CONNECTED:
-      break;
-    case PW_REMOTE_STATE_ERROR:
-      GST_ELEMENT_ERROR (pwsrc, RESOURCE, FAILED,
-          ("remote error: %s", error), (NULL));
-      break;
-  }
-  pw_thread_loop_signal (pwsrc->main_loop, FALSE);
-}
-
 static gboolean
 copy_properties (GQuark field_id,
                  const GValue *value,
@@ -968,55 +916,33 @@ copy_properties (GQuark field_id,
   return TRUE;
 }
 
-static const struct pw_remote_events remote_events = {
-	PW_VERSION_REMOTE_EVENTS,
-	.state_changed = on_remote_state_changed,
-};
-
 static const struct pw_stream_events stream_events = {
-	PW_VERSION_STREAM_EVENTS,
-	.state_changed = on_state_changed,
-	.format_changed = on_format_changed,
-	.add_buffer = on_add_buffer,
-	.remove_buffer = on_remove_buffer,
-	.process = on_process,
+        PW_VERSION_STREAM_EVENTS,
+        .state_changed = on_state_changed,
+        .param_changed = on_param_changed,
+        .add_buffer = on_add_buffer,
+        .remove_buffer = on_remove_buffer,
+        .process = on_process,
 };
 
 static gboolean
 gst_pipewire_src_open (GstPipeWireSrc * pwsrc)
 {
   struct pw_properties *props;
-  const char *error = NULL;
 
-  if (pw_thread_loop_start (pwsrc->main_loop) < 0)
+  if (pw_thread_loop_start (pwsrc->loop) < 0)
     goto mainloop_failed;
 
-  pw_thread_loop_lock (pwsrc->main_loop);
-  if ((pwsrc->remote = pw_remote_new (pwsrc->core, NULL, 0)) == NULL)
-    goto no_remote;
-
-  pw_remote_add_listener (pwsrc->remote,
-			  &pwsrc->remote_listener,
-			  &remote_events, pwsrc);
+  pw_thread_loop_lock (pwsrc->loop);
 
   if (pwsrc->fd == -1)
-    pw_remote_connect (pwsrc->remote);
+    pwsrc->core = pw_context_connect (pwsrc->context, NULL, 0);
   else
-    pw_remote_connect_fd (pwsrc->remote, pwsrc->fd);
+    pwsrc->core = pw_context_connect_fd (pwsrc->context, dup(pwsrc->fd), NULL, 0);
 
-  while (TRUE) {
-    enum pw_remote_state state = pw_remote_get_state(pwsrc->remote, &error);
-
-    GST_DEBUG ("waiting for CONNECTED, now %s", pw_remote_state_as_string (state));
-    if (state == PW_REMOTE_STATE_CONNECTED)
-      break;
-
-    if (state == PW_REMOTE_STATE_ERROR)
+  if (pwsrc->core == NULL)
       goto connect_error;
 
-    pw_thread_loop_wait (pwsrc->main_loop);
-  }
-
   if (pwsrc->properties) {
     props = pw_properties_new (NULL, NULL);
     gst_structure_foreach (pwsrc->properties, copy_properties, props);
@@ -1024,18 +950,19 @@ gst_pipewire_src_open (GstPipeWireSrc * pwsrc)
     props = NULL;
   }
 
-  if ((pwsrc->stream = pw_stream_new (pwsrc->remote, pwsrc->client_name, props)) == NULL)
+  if ((pwsrc->stream = pw_stream_new (pwsrc->core,
+                                  pwsrc->client_name, props)) == NULL)
     goto no_stream;
 
 
   pw_stream_add_listener(pwsrc->stream,
-			 &pwsrc->stream_listener,
-			 &stream_events,
-			 pwsrc);
+                         &pwsrc->stream_listener,
+                         &stream_events,
+                         pwsrc);
 
 
   pwsrc->clock = gst_pipewire_clock_new (pwsrc->stream, pwsrc->last_time);
-  pw_thread_loop_unlock (pwsrc->main_loop);
+  pw_thread_loop_unlock (pwsrc->loop);
 
   return TRUE;
 
@@ -1045,21 +972,16 @@ mainloop_failed:
     GST_ELEMENT_ERROR (pwsrc, RESOURCE, FAILED, ("error starting mainloop"), (NULL));
     return FALSE;
   }
-no_remote:
-  {
-    GST_ELEMENT_ERROR (pwsrc, RESOURCE, FAILED, ("can't create remote"), (NULL));
-    pw_thread_loop_unlock (pwsrc->main_loop);
-    return FALSE;
-  }
 connect_error:
   {
-    pw_thread_loop_unlock (pwsrc->main_loop);
+    GST_ELEMENT_ERROR (pwsrc, RESOURCE, FAILED, ("can't connect"), (NULL));
+    pw_thread_loop_unlock (pwsrc->loop);
     return FALSE;
   }
 no_stream:
   {
     GST_ELEMENT_ERROR (pwsrc, RESOURCE, FAILED, ("can't create stream"), (NULL));
-    pw_thread_loop_unlock (pwsrc->main_loop);
+    pw_thread_loop_unlock (pwsrc->loop);
     return FALSE;
   }
 }
@@ -1069,7 +991,7 @@ gst_pipewire_src_close (GstPipeWireSrc * pwsrc)
 {
   clear_queue (pwsrc);
 
-  pw_thread_loop_stop (pwsrc->main_loop);
+  pw_thread_loop_stop (pwsrc->loop);
 
   pwsrc->last_time = gst_clock_get_time (pwsrc->clock);
 
@@ -1084,9 +1006,8 @@ gst_pipewire_src_close (GstPipeWireSrc * pwsrc)
   pw_stream_destroy (pwsrc->stream);
   pwsrc->stream = NULL;
 
-  pw_remote_destroy (pwsrc->remote);
-  pwsrc->remote = NULL;
-
+  pw_core_disconnect (pwsrc->core);
+  pwsrc->core = NULL;
 }
 
 static GstStateChangeReturn
