@@ -63,6 +63,11 @@ static const struct spa_dict_item module_props[] = {
 	{ PW_KEY_MODULE_VERSION, PACKAGE_VERSION },
 };
 
+/* Required for s390x */
+#ifndef SO_PEERSEC
+#define SO_PEERSEC 31
+#endif
+
 static bool debug_messages = 0;
 
 #define LOCK_SUFFIX     ".lock"
@@ -1041,7 +1046,7 @@ error:
 	return NULL;
 }
 
-const static struct pw_protocol_implementaton protocol_impl = {
+static const struct pw_protocol_implementaton protocol_impl = {
 	PW_VERSION_PROTOCOL_IMPLEMENTATION,
 	.new_client = impl_new_client,
 	.add_server = impl_add_server,
@@ -1100,7 +1105,7 @@ static int impl_ext_end_resource(struct pw_resource *resource,
 	struct pw_impl_client *client = resource->client;
 	return client->send_seq = pw_protocol_native_connection_end(data->connection, builder);
 }
-const static struct pw_protocol_native_ext protocol_ext_impl = {
+static const struct pw_protocol_native_ext protocol_ext_impl = {
 	PW_VERSION_PROTOCOL_NATIVE_EXT,
 	.begin_proxy = impl_ext_begin_proxy,
 	.add_proxy_fd = impl_ext_add_proxy_fd,
