@@ -29,6 +29,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <alloca.h>
+#include <getopt.h>
 
 #include <spa/utils/result.h>
 #include <spa/debug/pod.h>
@@ -51,6 +52,8 @@ struct data {
 	struct remote_data *current;
 
 	struct pw_map vars;
+	unsigned int interactive:1;
+	unsigned int monitoring:1;
 };
 
 struct global {
@@ -276,11 +279,13 @@ static void on_core_info(void *_data, const struct pw_core_info *info)
 	struct remote_data *rd = _data;
 	free(rd->name);
 	rd->name = info->name ? strdup(info->name) : NULL;
-	fprintf(stdout, "remote %d is named '%s'\n", rd->id, rd->name);
+	if (rd->data->interactive)
+		fprintf(stdout, "remote %d is named '%s'\n", rd->id, rd->name);
 }
 
 static void show_prompt(struct remote_data *rd)
 {
+	rd->data->monitoring = true;
 	fprintf(stdout, "%s>>", rd->name);
 	fflush(stdout);
 }
@@ -288,9 +293,14 @@ static void show_prompt(struct remote_data *rd)
 static void on_core_done(void *_data, uint32_t id, int seq)
 {
 	struct remote_data *rd = _data;
+	struct data *d = rd->data;
 
-	if (seq == rd->prompt_pending)
-		show_prompt(rd);
+	if (seq == rd->prompt_pending) {
+		if (d->interactive)
+			show_prompt(rd);
+		else
+			pw_main_loop_quit(d->loop);
+	}
 }
 
 static int print_global(void *obj, void *data)
@@ -333,8 +343,10 @@ static void registry_event_global(void *data, uint32_t id,
 	global->version = version;
 	global->properties = props ? pw_properties_new_dict(props) : NULL;
 
-	fprintf(stdout, "remote %d added global: ", rd->id);
-	print_global(global, NULL);
+	if (rd->data->monitoring) {
+		fprintf(stdout, "remote %d added global: ", rd->id);
+		print_global(global, NULL);
+	}
 
 	size = pw_map_get_size(&rd->globals);
 	while (id > size)
@@ -344,7 +356,8 @@ static void registry_event_global(void *data, uint32_t id,
 	/* immediately bind the object always */
 	ret = bind_global(rd, global, &error);
 	if (!ret) {
-		fprintf(stdout, "Error: \"%s\"\n", error);
+		if (rd->data->interactive)
+			fprintf(stdout, "Error: \"%s\"\n", error);
 		free(error);
 	}
 }
@@ -375,8 +388,11 @@ static void registry_event_global_remove(void *data, uint32_t id)
 		return;
 	}
 
-	fprintf(stdout, "remote %d removed global: ", rd->id);
-	print_global(global, NULL);
+	if (rd->data->monitoring) {
+		fprintf(stdout, "remote %d removed global: ", rd->id);
+		print_global(global, NULL);
+	}
+
 	destroy_global(global, rd);
 }
 
@@ -434,7 +450,7 @@ static bool do_connect(struct data *data, const char *cmd, char *args, char **er
 	struct pw_core *core;
 	struct remote_data *rd;
 
-	n = pw_split_ip(args, WHITESPACE, 1, a);
+	n = args ? pw_split_ip(args, WHITESPACE, 1, a) : 0;
 	if (n == 1) {
 		props = pw_properties_new(PW_KEY_REMOTE_NAME, a[0], NULL);
 	}
@@ -451,7 +467,9 @@ static bool do_connect(struct data *data, const char *cmd, char *args, char **er
 	rd->id = pw_map_insert_new(&data->vars, rd);
 	spa_list_append(&data->remotes, &rd->link);
 
-	fprintf(stdout, "%d = @remote:%p\n", rd->id, rd->core);
+	if (rd->data->interactive)
+		fprintf(stdout, "%d = @remote:%p\n", rd->id, rd->core);
+
 	data->current = rd;
 
 	pw_core_add_listener(rd->core,
@@ -776,8 +794,9 @@ static void event_param(void *object, int seq, uint32_t id,
         struct proxy_data *data = object;
 	struct remote_data *rd = data->rd;
 
-	fprintf(stdout, "remote %d object %d param %d index %d\n",
-			rd->id, data->global->id, id, index);
+	if (rd->data->interactive)
+		fprintf(stdout, "remote %d object %d param %d index %d\n",
+				rd->id, data->global->id, id, index);
 
 	if (spa_pod_is_object_type(param, SPA_TYPE_OBJECT_Format))
 		spa_debug_format(2, NULL, param);
@@ -1183,6 +1202,8 @@ static bool bind_global(struct remote_data *rd, struct global *global, char **er
 
 	global->proxy = proxy;
 
+	rd->prompt_pending = pw_core_sync(rd->core, 0, 0);
+
 	return true;
 }
 
@@ -2722,14 +2743,60 @@ static void do_quit(void *data, int signal_number)
 	pw_main_loop_quit(d->loop);
 }
 
+static void show_help(struct data *data, const char *name)
+{
+        fprintf(stdout, "%s [options] [command]\n"
+             "  -h, --help                            Show this help\n"
+             "  -v, --version                         Show version\n"
+             "  -d, --daemon                          Start as daemon (Default false)\n"
+             "  -r, --remote                          Remote daemon name\n\n",
+	     name);
+
+	do_help(data, "help", "", NULL);
+}
+
 int main(int argc, char *argv[])
 {
 	struct data data = { 0 };
 	struct pw_loop *l;
+	char *opt_remote = NULL;
 	char *error;
+	bool daemon = false;
+	static const struct option long_options[] = {
+		{"help",	0, NULL, 'h'},
+		{"version",	0, NULL, 'v'},
+		{"daemon",	0, NULL, 'd'},
+		{"remote",	1, NULL, 'r'},
+		{NULL,		0, NULL, 0}
+	};
+	int c, i;
 
 	pw_init(&argc, &argv);
 
+	while ((c = getopt_long(argc, argv, "hvdr:", long_options, NULL)) != -1) {
+		switch (c) {
+		case 'h':
+			show_help(&data, argv[0]);
+			return 0;
+		case 'v':
+			fprintf(stdout, "%s\n"
+				"Compiled with libpipewire %s\n"
+				"Linked with libpipewire %s\n",
+				argv[0],
+				pw_get_headers_version(),
+				pw_get_library_version());
+			return 0;
+		case 'd':
+			daemon = true;
+			break;
+		case 'r':
+			opt_remote = optarg;
+			break;
+		default:
+			return -1;
+		}
+	}
+
 	data.loop = pw_main_loop_new(NULL);
 	l = pw_main_loop_get_loop(data.loop);
 	pw_loop_add_signal(l, SIGINT, do_quit, &data);
@@ -2738,18 +2805,45 @@ int main(int argc, char *argv[])
 	spa_list_init(&data.remotes);
 	pw_map_init(&data.vars, 64, 16);
 
-	data.context = pw_context_new(l, pw_properties_new(PW_KEY_CORE_DAEMON, "true", NULL), 0);
+	data.context = pw_context_new(l,
+			pw_properties_new(
+				PW_KEY_CORE_DAEMON, daemon ? "true" : NULL,
+				NULL),
+			0);
 
 	pw_context_load_module(data.context, "libpipewire-module-link-factory", NULL, NULL);
 
-	pw_loop_add_io(l, STDIN_FILENO, SPA_IO_IN|SPA_IO_HUP, false, do_input, &data);
+	do_connect(&data, "connect", opt_remote, &error);
 
-	fprintf(stdout, "Welcome to PipeWire version %s. Type 'help' for usage.\n",
-			pw_get_library_version());
+	if (optind == argc) {
+		data.interactive = true;
 
-	do_connect(&data, "connect", "internal", &error);
+		pw_loop_add_io(l, STDIN_FILENO, SPA_IO_IN|SPA_IO_HUP, false, do_input, &data);
 
-	pw_main_loop_run(data.loop);
+		fprintf(stdout, "Welcome to PipeWire version %s. Type 'help' for usage.\n",
+				pw_get_library_version());
+
+		pw_main_loop_run(data.loop);
+	} else {
+		char buf[4096], *p, *error;
+
+		p = buf;
+		for (i = optind; i < argc; i++) {
+			p = stpcpy(p, argv[i]);
+			p = stpcpy(p, " ");
+		}
+
+		pw_main_loop_run(data.loop);
+
+		if (!parse(&data, buf, p - buf, &error)) {
+			fprintf(stdout, "Error: \"%s\"\n", error);
+			free(error);
+		}
+		if (data.current) {
+			data.current->prompt_pending = pw_core_sync(data.current->core, 0, 0);
+			pw_main_loop_run(data.loop);
+		}
+	}
 
 	pw_context_destroy(data.context);
 	pw_main_loop_destroy(data.loop);
